@@ -59,8 +59,9 @@
 #include "goto.h"
 #include "print.h"
 #include "dialogs/cal-prefs-dialog.h"
-#include "dialogs/new-calendar.h"
 #include "itip-utils.h"
+#include "e-pub-utils.h"
+#include "e-cal-list-view.h"
 #include "evolution-shell-component-utils.h"
 
 /* Focusing information for the calendar view.  We have to keep track of this
@@ -72,15 +73,16 @@ typedef struct {
 } FocusData;
 
 static void
-file_new_calendar_cb (BonoboUIComponent *uic, gpointer data, const char *path)
+file_open_event_cb (BonoboUIComponent *uic, gpointer data, const char *path)
 {
 	GnomeCalendar *gcal;
 
 	gcal = GNOME_CALENDAR (data);
 
-	new_calendar_dialog (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (gcal))));
+	e_calendar_view_open_event (gnome_calendar_get_current_view_widget (gcal));
 }
 
+
 /* Prints the calendar at its current view and time range */
 static void
 print (GnomeCalendar *gcal, gboolean preview)
@@ -88,7 +90,9 @@ print (GnomeCalendar *gcal, gboolean preview)
 	time_t start;
 	GnomeCalendarViewType view_type;
 	PrintView print_view;
-
+	ECalListView *list_view;
+	ETable *etable;
+	
 	gnome_calendar_get_current_time_range (gcal, &start, NULL);
 	view_type = gnome_calendar_get_view (gcal);
 
@@ -106,6 +110,12 @@ print (GnomeCalendar *gcal, gboolean preview)
 		print_view = PRINT_VIEW_MONTH;
 		break;
 
+	case GNOME_CAL_LIST_VIEW:
+		list_view = E_CAL_LIST_VIEW (gnome_calendar_get_current_view_widget (gcal));
+		etable = e_table_scrolled_get_table (list_view->table_scrolled);
+		print_table (etable, _("Calendar"), preview);
+		return;
+
 	default:
 		g_assert_not_reached ();
 		return;
@@ -319,36 +329,7 @@ delete_occurrence_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
 static void
 publish_freebusy_cmd (BonoboUIComponent *uic, gpointer data, const gchar *path)
 {
-	GnomeCalendar *gcal;
-	GList *client_list, *cl;
-	GList *comp_list = NULL;
-	icaltimezone *utc;
-	time_t start = time (NULL), end;
-
-	gcal = GNOME_CALENDAR (data);
-
-	utc = icaltimezone_get_utc_timezone ();
-	start = time_day_begin_with_zone (start, utc);
-	end = time_add_week_with_zone (start, 6, utc);
-
-	/* FIXME Should we aggregate the data? */
-	client_list = e_cal_model_get_client_list (gnome_calendar_get_calendar_model (gcal));
-	for (cl = client_list; cl != NULL; cl = cl->next) {
-		if (e_cal_get_free_busy ((ECal *) cl->data, NULL, start, end, &comp_list, NULL)) {
-			GList *l;
-
-			for (l = comp_list; l; l = l->next) {
-				ECalComponent *comp = E_CAL_COMPONENT (l->data);
-				itip_send_comp (E_CAL_COMPONENT_METHOD_PUBLISH, comp, (ECal *) cl->data, NULL);
-
-				g_object_unref (comp);
-			}
-
-			g_list_free (comp_list);
-		}
-	}
-
-	g_list_free (client_list);
+	e_pub_publish (TRUE);
 }
 
 static void
@@ -522,6 +503,9 @@ calendar_control_sensitize_calendar_commands (BonoboControl *control, GnomeCalen
 	uic = bonobo_control_get_ui_component (control);
 	g_assert (uic != NULL);
 
+	if (bonobo_ui_component_get_container (uic) == CORBA_OBJECT_NIL)
+		return;
+	
 	view = gnome_calendar_get_current_view_widget (gcal);
 	list = e_calendar_view_get_selected_events (E_CALENDAR_VIEW (view));
 
@@ -538,6 +522,9 @@ calendar_control_sensitize_calendar_commands (BonoboControl *control, GnomeCalen
 			read_only = TRUE;
 	}
 
+	bonobo_ui_component_set_prop (uic, "/commands/EventOpen", "sensitive",
+				      n_selected != 1 ? "0" : "1",
+				      NULL);
 	bonobo_ui_component_set_prop (uic, "/commands/Cut", "sensitive",
 				      n_selected == 0 || read_only ? "0" : "1",
 				      NULL);
@@ -691,7 +678,7 @@ gcal_taskpad_focus_change_cb (GnomeCalendar *gcal, gboolean in, gpointer data)
 
 
 static BonoboUIVerb verbs [] = {
-	BONOBO_UI_VERB ("NewCalendar", file_new_calendar_cb),
+	BONOBO_UI_VERB ("EventOpen", file_open_event_cb),
 	BONOBO_UI_VERB ("CalendarPrint", file_print_cb),
 	BONOBO_UI_VERB ("CalendarPrintPreview", file_print_preview_cb),
 
