@@ -31,6 +31,7 @@
 #include <bonobo/bonobo-i18n.h>
 #include <bonobo/bonobo-exception.h>
 #include "e-pub-utils.h"
+#include "e-cal-view.h"
 #include "calendar-config-keys.h"
 #include "calendar-config.h"
 #include "calendar-component.h"
@@ -58,6 +59,7 @@
 static BonoboObjectClass *parent_class = NULL;
 
 struct _CalendarComponentPrivate {
+	char *base_directory;
 	char *config_directory;
 
 	GConfClient *gconf_client;
@@ -112,26 +114,6 @@ is_in_uids (GSList *uids, ESource *source)
 	return FALSE;
 }
 
-static ESource *
-find_first_source (ESourceList *source_list)
-{
-	GSList *groups, *sources, *l, *m;
-			
-	groups = e_source_list_peek_groups (source_list);
-	for (l = groups; l; l = l->next) {
-		ESourceGroup *group = l->data;
-				
-		sources = e_source_group_peek_sources (group);
-		for (m = sources; m; m = m->next) {
-			ESource *source = m->data;
-
-			return source;
-		}				
-	}
-
-	return NULL;
-}
-
 static void
 update_uris_for_selection (CalendarComponent *calendar_component)
 {
@@ -227,7 +209,7 @@ static void
 update_primary_selection (CalendarComponent *calendar_component)
 {
 	CalendarComponentPrivate *priv;
-	ESource *source;
+	ESource *source = NULL;
 	char *uid;
 
 	priv = calendar_component->priv;
@@ -236,11 +218,11 @@ update_primary_selection (CalendarComponent *calendar_component)
 	if (uid) {
 		source = e_source_list_peek_source_by_uid (priv->source_list, uid);
 		g_free (uid);
+	}
 	
+	if (source) {
 		e_source_selector_set_primary_selection (E_SOURCE_SELECTOR (priv->source_selector), source);
 	} else {
-		ESource *source;
-		
 		/* Try to create a default if there isn't one */
 		source = e_source_list_peek_source_any (priv->source_list);
 		if (source)
@@ -248,48 +230,6 @@ update_primary_selection (CalendarComponent *calendar_component)
 	}
 }
 
-/* FIXME This is duplicated from comp-editor-factory.c, should it go in comp-util? */
-static ECalComponent *
-get_default_event (ECal *client, gboolean all_day) 
-{
-	ECalComponent *comp;
-	struct icaltimetype itt;
-	ECalComponentDateTime dt;
-	char *location;
-	icaltimezone *zone;
-
-	comp = cal_comp_event_new_with_defaults (client);
-
-	g_return_val_if_fail (comp, NULL);
-	
-	location = calendar_config_get_timezone ();
-	zone = icaltimezone_get_builtin_timezone (location);
-
-	if (all_day) {
-		itt = icaltime_from_timet_with_zone (time (NULL), 1, zone);
-
-		dt.value = &itt;
-		dt.tzid = icaltimezone_get_tzid (zone);
-		
-		e_cal_component_set_dtstart (comp, &dt);
-		e_cal_component_set_dtend (comp, &dt);		
-	} else {
-		itt = icaltime_current_time_with_zone (zone);
-		icaltime_adjust (&itt, 0, 1, -itt.minute, -itt.second);
-		
-		dt.value = &itt;
-		dt.tzid = icaltimezone_get_tzid (zone);
-		
-		e_cal_component_set_dtstart (comp, &dt);
-		icaltime_adjust (&itt, 0, 1, 0, 0);
-		e_cal_component_set_dtend (comp, &dt);
-	}
-
-	e_cal_component_commit_sequence (comp);
-
-	return comp;
-}
-
 /* Callbacks.  */
 static void
 add_popup_menu_item (GtkMenu *menu, const char *label, const char *pixmap,
@@ -361,14 +301,24 @@ delete_calendar_cb (GtkWidget *widget, CalendarComponent *comp)
 		_("Calendar '%s' will be removed. Are you sure you want to continue?"),
 		e_source_peek_name (selected_source));
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES) {
-		if (e_source_selector_source_is_selected (E_SOURCE_SELECTOR (priv->source_selector),
-							  selected_source))
-			e_source_selector_unselect_source (E_SOURCE_SELECTOR (priv->source_selector),
-							   selected_source);
-		
-		e_source_group_remove_source (e_source_peek_group (selected_source), selected_source);
+		ECal *cal;
+		char *uri;
 
-		/* FIXME: remove the calendar.ics file and the directory */
+		/* first, ask the backend to remove the calendar */
+		uri = e_source_get_uri (selected_source);
+		cal = e_cal_model_get_client_for_uri (gnome_calendar_get_calendar_model (priv->calendar), uri);
+		g_free (uri);
+		if (cal) {
+			if (e_cal_remove_calendar (cal, NULL)) {
+				if (e_source_selector_source_is_selected (E_SOURCE_SELECTOR (priv->source_selector),
+									  selected_source))
+					e_source_selector_unselect_source (E_SOURCE_SELECTOR (priv->source_selector),
+									   selected_source);
+		
+				e_source_group_remove_source (e_source_peek_group (selected_source), selected_source);
+				e_source_list_sync (priv->source_list, NULL);
+			}
+		}
 	}
 
 	gtk_widget_destroy (dialog);
@@ -422,6 +372,13 @@ static void
 primary_source_selection_changed_cb (ESourceSelector *selector,
 				     CalendarComponent *calendar_component)
 {
+	CalendarComponentPrivate *priv = calendar_component->priv;
+
+	if (priv->create_ecal) {
+		g_object_unref (priv->create_ecal);
+		priv->create_ecal = NULL;
+	}
+
 	update_uri_for_primary_selection (calendar_component);
 }
 
@@ -503,8 +460,8 @@ impl_finalize (GObject *object)
 {
 	CalendarComponentPrivate *priv = CALENDAR_COMPONENT (object)->priv;
 
+	g_free (priv->base_directory);
 	g_free (priv->config_directory);
-
 	g_free (priv);
 
 	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -520,85 +477,9 @@ impl_upgradeFromVersion (PortableServer_Servant servant,
 			 CORBA_short revision,
 			 CORBA_Environment *ev)
 {
-	CalendarComponentPrivate *priv;
-	GSList *groups;
-	ESourceGroup *group;
-	ESource *source;
-	char *base_uri, *new_dir;
 	CalendarComponent *calendar_component = CALENDAR_COMPONENT (bonobo_object_from_servant (servant));
 
-	priv = calendar_component->priv;
-
-	base_uri = g_build_filename (g_get_home_dir (),
-				     "/.evolution/calendar/local/OnThisComputer/",
-				     NULL);
-
-	/* create default calendars if there are no groups */
-	groups = e_source_list_peek_groups (priv->source_list);
-	if (!groups) {
-		/* create the local source group */
-		group = e_source_group_new (_("On This Computer"), base_uri);
-		e_source_list_add_group (priv->source_list, group, -1);
-
-		/* create the remote source group */
-		group = e_source_group_new (_("On The Web"), "webcal://");
-		e_source_list_add_group (priv->source_list, group, -1);
- 		e_source_list_sync (priv->source_list, NULL);
-	}
-
-	if (major == 1 && minor <= 4) {
-		group = e_source_list_peek_group_by_name (priv->source_list, _("On This Computer"));
-
-		/* migrate calendars from 1.4 setup */
-		if (!migrate_old_calendars (group)) {
-			/* create default calendars */
-			new_dir = g_build_filename (base_uri, "Personal/", NULL);
-			if (!e_mkdir_hier (new_dir, 0700)) {
-				source = e_source_new (_("Personal"), "Personal");
-				e_source_group_add_source (group, source, -1);
-			}
-			g_free (new_dir);
-
-			new_dir = g_build_filename (base_uri, "Work/", NULL);
-			if (!e_mkdir_hier (new_dir, 0700)) {
-				source = e_source_new (_("Work"), "Work");
-				e_source_group_add_source (group, source, -1);
-			}
-			g_free (new_dir);
-		}
-
-		g_free (base_uri);
-
-		/* create the remote source group */
-		group = e_source_group_new (_("On The Web"), "webcal://");
-		e_source_list_add_group (priv->source_list, group, -1);
-
- 		e_source_list_sync (priv->source_list, NULL);
-	}
-
-	/* create calendar for birthdays & anniversaries */
-	if ((major < 0) ||
-	    ((major == 1) && (minor < 5)) ||
-	    ((major == 1) && (minor == 5) && (revision < 2))) {		
-		group = e_source_group_new (_("Contacts"), "contacts://");
-		source = e_source_new (_("Birthdays & Anniversaries"), "/");
-		e_source_group_add_source (group, source, -1);
-		e_source_group_set_readonly (group, TRUE);
-		e_source_list_add_group (priv->source_list, group, -1);
- 		e_source_list_sync (priv->source_list, NULL);
-	}
-
- 	/* create calendar for birthdays & anniversaries */
- 	if ((major < 0) ||
- 	    ((major == 1) && (minor < 5)) ||
-	    ((major == 1) && (minor == 5) && (revision < 2))) { 		
- 		group = e_source_group_new (_("Birthdays"), "contacts://");
- 		source = e_source_new (_("Birthdays & Anniversaries"), "/");
- 		e_source_group_add_source (group, source, -1);
- 		e_source_group_set_readonly (group, TRUE);
- 		e_source_list_add_group (priv->source_list, group, -1);
- 		e_source_list_sync (priv->source_list, NULL);
-  	}
+	migrate_calendars (calendar_component, major, minor, revision);
 
 	return CORBA_TRUE;
 }
@@ -837,56 +718,57 @@ setup_create_ecal (CalendarComponent *calendar_component)
 	return TRUE;
 }
 
+static void
+create_new_event (CalendarComponent *calendar_component, gboolean is_allday, gboolean is_meeting, CORBA_Environment *ev)
+{
+	CalendarComponentPrivate *priv = calendar_component->priv;
+	gboolean read_only;
+	ECalendarView *view;
+
+	if (!setup_create_ecal (calendar_component)) {
+		bonobo_exception_set (ev, ex_GNOME_Evolution_Component_Failed);
+		return;
+	}
+	if (!e_cal_is_read_only (priv->create_ecal, &read_only, NULL) || read_only)
+		return;
+
+	if (priv->calendar && (view = E_CALENDAR_VIEW (gnome_calendar_get_current_view_widget (priv->calendar))))
+		e_calendar_view_new_appointment_full (view, is_allday, is_meeting);
+	else {
+		ECalComponent *comp;
+		EventEditor *editor;
+
+		editor = event_editor_new (priv->create_ecal);
+		comp = cal_comp_event_new_with_current_time (priv->create_ecal, is_allday);
+
+		comp_editor_edit_comp (COMP_EDITOR (editor), comp);
+		if (is_meeting)
+			event_editor_show_meeting (editor);
+		comp_editor_focus (COMP_EDITOR (editor));
+
+		e_comp_editor_registry_add (comp_editor_registry, COMP_EDITOR (editor), TRUE);
+	}
+}
+
 static void
 impl_requestCreateItem (PortableServer_Servant servant,
 			const CORBA_char *item_type_name,
 			CORBA_Environment *ev)
 {
 	CalendarComponent *calendar_component = CALENDAR_COMPONENT (bonobo_object_from_servant (servant));
-	CalendarComponentPrivate *priv;
-	ECalComponent *comp;
-	EventEditor *editor;
-	gboolean is_meeting = FALSE;
-	gboolean read_only = FALSE;
 	
-	priv = calendar_component->priv;
-
-	if (!setup_create_ecal (calendar_component)) {
-		bonobo_exception_set (ev, ex_GNOME_Evolution_Component_Failed);
-		return;
-
-	e_cal_is_read_only (priv->create_ecal, &read_only, NULL);
-	if (read_only)
-		return;
-	
-	}
-	
-	editor = event_editor_new (priv->create_ecal);
-	
-	if (strcmp (item_type_name, CREATE_EVENT_ID) == 0) {
-		comp = get_default_event (priv->create_ecal, FALSE);
- 	} else if (strcmp (item_type_name, CREATE_ALLDAY_EVENT_ID) == 0) {
-		comp = get_default_event (priv->create_ecal, TRUE);
-	} else if (strcmp (item_type_name, CREATE_MEETING_ID) == 0) {
-		comp = get_default_event (priv->create_ecal, FALSE);
-		is_meeting = TRUE;
-	} else if (strcmp (item_type_name, CREATE_CALENDAR_ID) == 0) {
-		calendar_setup_new_calendar (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (priv->calendar))));
-		return;
-	} else {
+	if (strcmp (item_type_name, CREATE_EVENT_ID) == 0)
+		create_new_event (calendar_component, FALSE, FALSE, ev);
+ 	else if (strcmp (item_type_name, CREATE_ALLDAY_EVENT_ID) == 0)
+		create_new_event (calendar_component, TRUE, FALSE, ev);
+	else if (strcmp (item_type_name, CREATE_MEETING_ID) == 0)
+		create_new_event (calendar_component, FALSE, TRUE, ev);
+	else if (strcmp (item_type_name, CREATE_CALENDAR_ID) == 0)
+		calendar_setup_new_calendar (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (calendar_component->priv->calendar))));
+	else
 		bonobo_exception_set (ev, ex_GNOME_Evolution_Component_UnknownType);
-		return;
-	}	
-
-	comp_editor_edit_comp (COMP_EDITOR (editor), comp);
-	if (is_meeting)
-		event_editor_show_meeting (editor);
-	comp_editor_focus (COMP_EDITOR (editor));
-
-	e_comp_editor_registry_add (comp_editor_registry, COMP_EDITOR (editor), TRUE);
 }
 
-
 /* Initialization.  */
 
 static void
@@ -913,6 +795,7 @@ calendar_component_init (CalendarComponent *component)
 
 	priv = g_new0 (CalendarComponentPrivate, 1);
 
+	priv->base_directory = g_build_filename (g_get_home_dir (), ".evolution", NULL);
 	priv->config_directory = g_build_filename (g_get_home_dir (),
 						   ".evolution", "calendar", "config",
 						   NULL);
@@ -952,12 +835,30 @@ calendar_component_peek (void)
 	return component;
 }
 
+const char *
+calendar_component_peek_base_directory (CalendarComponent *component)
+{
+	return component->priv->base_directory;
+}
+
 const char *
 calendar_component_peek_config_directory (CalendarComponent *component)
 {
 	return component->priv->config_directory;
 }
 
+ESourceList *
+calendar_component_peek_source_list (CalendarComponent *component)
+{
+	return component->priv->source_list;
+}
+
+ESourceSelector *
+calendar_component_peek_source_selector (CalendarComponent *component)
+{
+	return component->priv->source_selector;
+}
+
 EActivityHandler *
 calendar_component_peek_activity_handler (CalendarComponent *component)
 {
