@@ -1,24 +1,26 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; fill-column: 160 -*-
- * 
- * Authors: Michael Zucchi <notzed@ximian.com>
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/*
+ *  Authors: Michael Zucchi <notzed@ximian.com>
  *
- * Copyright (C) 1999, 2000 Ximian Inc.
+ *  Copyright 1999-2003 Ximian, Inc. (www.ximian.com)
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU General Public 
- * License as published by the Free Software Foundation.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -39,7 +41,6 @@
 
 #include "camel-local-folder.h"
 #include "camel-local-store.h"
-#include "string-utils.h"
 #include "camel-stream-fs.h"
 #include "camel-local-summary.h"
 #include "camel-data-wrapper.h"
@@ -58,7 +59,8 @@
 #define PATH_MAX _POSIX_PATH_MAX
 #endif
 
-static CamelFolderClass *parent_class = NULL;
+static CamelFolderClass *parent_class;
+static GSList *local_folder_properties;
 
 /* Returns the class for a CamelLocalFolder */
 #define CLOCALF_CLASS(so) CAMEL_LOCAL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS(so))
@@ -66,10 +68,14 @@ static CamelFolderClass *parent_class = NULL;
 #define CLOCALS_CLASS(so) CAMEL_STORE_CLASS (CAMEL_OBJECT_GET_CLASS(so))
 
 static int local_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args);
+static int local_setv(CamelObject *object, CamelException *ex, CamelArgV *args);
 
 static int local_lock(CamelLocalFolder *lf, CamelLockType type, CamelException *ex);
 static void local_unlock(CamelLocalFolder *lf);
 
+static char *local_get_full_path (const char *toplevel_dir, const char *full_name);
+static char *local_get_meta_path (const char *toplevel_dir, const char *full_name, const char *ext);
+
 static void local_refresh_info(CamelFolder *folder, CamelException *ex);
 
 static void local_sync(CamelFolder *folder, gboolean expunge, CamelException *ex);
@@ -90,12 +96,11 @@ camel_local_folder_class_init(CamelLocalFolderClass * camel_local_folder_class)
 	CamelFolderClass *camel_folder_class = CAMEL_FOLDER_CLASS(camel_local_folder_class);
 	CamelObjectClass *oklass = (CamelObjectClass *)camel_local_folder_class;
 
-	parent_class = CAMEL_FOLDER_CLASS(camel_type_get_global_classfuncs(camel_folder_get_type()));
-
 	/* virtual method definition */
 
 	/* virtual method overload */
 	oklass->getv = local_getv;
+	oklass->setv = local_setv;
 
 	camel_folder_class->refresh_info = local_refresh_info;
 	camel_folder_class->sync = local_sync;
@@ -108,6 +113,9 @@ camel_local_folder_class_init(CamelLocalFolderClass * camel_local_folder_class)
 	camel_folder_class->delete = local_delete;
 	camel_folder_class->rename = local_rename;
 
+	camel_local_folder_class->get_full_path = local_get_full_path;
+	camel_local_folder_class->get_meta_path = local_get_meta_path;
+	
 	camel_local_folder_class->lock = local_lock;
 	camel_local_folder_class->unlock = local_unlock;
 }
@@ -130,9 +138,7 @@ local_init(gpointer object, gpointer klass)
 	local_folder->search = NULL;
 
 	local_folder->priv = g_malloc0(sizeof(*local_folder->priv));
-#ifdef ENABLE_THREADS
 	local_folder->priv->search_lock = g_mutex_new();
-#endif
 }
 
 static void
@@ -163,25 +169,37 @@ local_finalize(CamelObject * object)
 	g_free(local_folder->index_path);
 
 	camel_folder_change_info_free(local_folder->changes);
-
-#ifdef ENABLE_THREADS
+	
 	g_mutex_free(local_folder->priv->search_lock);
-#endif
+	
 	g_free(local_folder->priv);
 }
 
-CamelType camel_local_folder_get_type(void)
+static CamelProperty local_property_list[] = {
+	{ CAMEL_LOCAL_FOLDER_INDEX_BODY, "index_body", N_("Index message body data") },
+};
+
+CamelType
+camel_local_folder_get_type(void)
 {
 	static CamelType camel_local_folder_type = CAMEL_INVALID_TYPE;
 
 	if (camel_local_folder_type == CAMEL_INVALID_TYPE) {
-		camel_local_folder_type = camel_type_register(CAMEL_FOLDER_TYPE, "CamelLocalFolder",
+		int i;
+
+		parent_class = (CamelFolderClass *)camel_folder_get_type();
+		camel_local_folder_type = camel_type_register(camel_folder_get_type(), "CamelLocalFolder",
 							     sizeof(CamelLocalFolder),
 							     sizeof(CamelLocalFolderClass),
 							     (CamelObjectClassInitFunc) camel_local_folder_class_init,
 							     NULL,
 							     (CamelObjectInitFunc) local_init,
 							     (CamelObjectFinalizeFunc) local_finalize);
+
+		for (i=0;i<sizeof(local_property_list)/sizeof(local_property_list[0]);i++) {
+			local_property_list[i].description = _(local_property_list[i].description);
+			local_folder_properties = g_slist_prepend(local_folder_properties, &local_property_list[i]);
+		}
 	}
 
 	return camel_local_folder_type;
@@ -193,7 +211,7 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 	CamelFolderInfo *fi;
 	CamelFolder *folder;
 	const char *root_dir_path, *name;
-	char *tmp;
+	char *tmp, *statepath;
 	char folder_path[PATH_MAX];
 	struct stat st;
 	int forceindex, len;
@@ -224,12 +242,22 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 		/* not really sure to do with these for now? */
 		lf->summary_path = g_strdup_printf("%s.ev-summary", tmp);
 		lf->index_path = g_strdup_printf("%s.ibex", tmp);
+		statepath = g_strdup_printf ("%s.cmeta", tmp);
 	} else {
-		lf->folder_path = g_strdup_printf("%s/%s", root_dir_path, full_name);
-		lf->summary_path = g_strdup_printf("%s/%s.ev-summary", root_dir_path, full_name);
-		lf->index_path = g_strdup_printf("%s/%s.ibex", root_dir_path, full_name);
+		lf->folder_path = CLOCALF_CLASS (lf)->get_full_path (root_dir_path, full_name);
+		lf->summary_path = CLOCALF_CLASS (lf)->get_meta_path (root_dir_path, full_name, ".ev-summary");
+		lf->index_path = CLOCALF_CLASS (lf)->get_meta_path (root_dir_path, full_name, ".ibex");
+		statepath = CLOCALF_CLASS (lf)->get_meta_path (root_dir_path, full_name, ".cmeta");
 	}
-	
+	camel_object_set(lf, NULL, CAMEL_OBJECT_STATE_FILE, statepath, NULL);
+	g_free (statepath);
+
+	lf->flags = flags;
+
+	if (camel_object_state_read(lf) == -1) {
+		/* FIXME: load defaults? */
+	}
+
 	/* follow any symlinks to the mailbox */
 	if (lstat (lf->folder_path, &st) != -1 && S_ISLNK (st.st_mode) &&
 	    realpath (lf->folder_path, folder_path) != NULL) {
@@ -243,7 +271,8 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 	   the old-format 'ibex' files that might be lying around */
 	unlink(lf->index_path);
 
-#if 0
+	/* FIXME: Need to run indexing off of the setv method */
+#if 1
 	forceindex = FALSE;
 #else
 	/* if we have no/invalid index file, force it */
@@ -266,8 +295,8 @@ camel_local_folder_construct(CamelLocalFolder *lf, CamelStore *parent_store, con
 			camel_text_index_remove(lf->index_path);
 		forceindex = FALSE;
 	}
-#endif
 	lf->flags = flags;
+#endif
 
 	folder->summary = (CamelFolderSummary *)CLOCALF_CLASS(lf)->create_summary(lf->summary_path, lf->folder_path, lf->index);
 	if (camel_local_summary_load((CamelLocalSummary *)folder->summary, forceindex, ex) == -1) {
@@ -327,7 +356,7 @@ static int
 local_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 {
 	CamelFolder *folder = (CamelFolder *)object;
-	int i, count=args->argc;
+	int i;
 	guint32 tag;
 
 	for (i=0;i<args->argc;i++) {
@@ -336,7 +365,6 @@ local_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 		tag = arg->tag;
 
 		switch (tag & CAMEL_ARG_TAG) {
-			/* CamelObject args */
 		case CAMEL_OBJECT_ARG_DESCRIPTION:
 			if (folder->description == NULL) {
 				char *tmp, *path;
@@ -367,18 +395,74 @@ local_getv(CamelObject *object, CamelException *ex, CamelArgGetV *args)
 			}
 			*arg->ca_str = folder->description;
 			break;
+
+		case CAMEL_OBJECT_ARG_PERSISTENT_PROPERTIES:
+		case CAMEL_FOLDER_ARG_PROPERTIES: {
+			CamelArgGetV props;
+
+			props.argc = 1;
+			props.argv[0] = *arg;
+			((CamelObjectClass *)parent_class)->getv(object, ex, &props);
+			*arg->ca_ptr = g_slist_concat(*arg->ca_ptr, g_slist_copy(local_folder_properties));
+
+			break; }
+
+		case CAMEL_LOCAL_FOLDER_ARG_INDEX_BODY:
+			/* FIXME: remove this from sotre flags */
+			*arg->ca_int = (((CamelLocalFolder *)folder)->flags & CAMEL_STORE_FOLDER_BODY_INDEX) != 0;
+			break;
+
 		default: skip:
-			count--;
 			continue;
 		}
 
 		arg->tag = (tag & CAMEL_ARG_TYPE) | CAMEL_ARG_IGNORE;
 	}
 
-	if (count)
-		return ((CamelObjectClass *)parent_class)->getv(object, ex, args);
+	return ((CamelObjectClass *)parent_class)->getv(object, ex, args);
+}
 
-	return 0;
+static int
+local_setv(CamelObject *object, CamelException *ex, CamelArgV *args)
+{
+	int i;
+	guint32 tag;
+
+	for (i=0;i<args->argc;i++) {
+		CamelArg *arg = &args->argv[i];
+
+		tag = arg->tag;
+
+		switch (tag & CAMEL_ARG_TAG) {
+		case CAMEL_LOCAL_FOLDER_ARG_INDEX_BODY:
+			/* FIXME: implement */
+			/* TODO: When turning on (off?) the index, we want to launch a task for it,
+			   and make sure we dont have multiple tasks doing the same job */
+			if (arg->ca_int)
+				((CamelLocalFolder *)object)->flags |= CAMEL_STORE_FOLDER_BODY_INDEX;
+			else
+				((CamelLocalFolder *)object)->flags &= ~CAMEL_STORE_FOLDER_BODY_INDEX;
+			break;
+		default:
+			continue;
+		}
+
+		arg->tag = (tag & CAMEL_ARG_TYPE) | CAMEL_ARG_IGNORE;
+	}
+
+	return ((CamelObjectClass *)parent_class)->setv(object, ex, args);
+}
+
+static char *
+local_get_full_path (const char *toplevel_dir, const char *full_name)
+{
+	return g_strdup_printf ("%s/%s", toplevel_dir, full_name);
+}
+
+static char *
+local_get_meta_path (const char *toplevel_dir, const char *full_name, const char *ext)
+{
+	return g_strdup_printf ("%s/%s%s", toplevel_dir, full_name, ext);
 }
 
 static int
@@ -418,6 +502,8 @@ local_sync(CamelFolder *folder, gboolean expunge, CamelException *ex)
 	if (camel_local_folder_lock(lf, CAMEL_LOCK_WRITE, ex) == -1)
 		return;
 
+	camel_object_state_write(lf);
+
 	/* if sync fails, we'll pass it up on exit through ex */
 	camel_local_summary_sync((CamelLocalSummary *)folder->summary, expunge, lf->changes, ex);
 	camel_local_folder_unlock(lf);
@@ -442,8 +528,9 @@ static void
 local_delete(CamelFolder *folder)
 {
 	CamelLocalFolder *lf = (CamelLocalFolder *)folder;
-
-	camel_index_delete(lf->index);
+	
+	if (lf->index)
+		camel_index_delete(lf->index);
 
 	parent_class->delete(folder);
 }
