@@ -42,7 +42,6 @@
 #include "mail-tools.h"
 #include "mail-config.h"
 
-#include <e-util/e-passwords.h>
 #include <e-util/e-dialog-utils.h>
 
 #include <camel/camel-mime-message.h>
@@ -100,6 +99,8 @@ struct _EMFolderBrowserPrivate {
 
 	guint vpane_resize_id;
 	guint list_built_id;	/* hook onto list-built for delayed 'select first unread' stuff */
+	
+	char *select_uid;
 };
 
 static void emfb_activate(EMFolderView *emfv, BonoboUIComponent *uic, int state);
@@ -107,11 +108,14 @@ static void emfb_set_folder(EMFolderView *emfv, CamelFolder *folder, const char
 
 /* FilterBar stuff ... */
 static void emfb_search_config_search(EFilterBar *efb, FilterRule *rule, int id, const char *query, void *data);
-static void emfb_search_menu_activated(ESearchBar *esb, int id, EMFolderBrowser *fb);
+static void emfb_search_menu_activated(ESearchBar *esb, int id, EMFolderBrowser *emfb);
 static void emfb_search_search_activated(ESearchBar *esb, EMFolderBrowser *emfb);
-static void emfb_search_query_changed(ESearchBar *esb, EMFolderBrowser *fb);
+static void emfb_search_query_changed(ESearchBar *esb, EMFolderBrowser *emfb);
 
-static int emfb_list_key_press(ETree *tree, int row, ETreePath path, int col, GdkEvent *ev, EMFolderBrowser *fb);
+static int emfb_list_key_press(ETree *tree, int row, ETreePath path, int col, GdkEvent *ev, EMFolderBrowser *emfb);
+static void emfb_list_message_selected (MessageList *ml, const char *uid, EMFolderBrowser *emfb);
+
+static void emfb_create_view_menus(EMFolderBrowser *emfb, BonoboUIComponent *uic);
 
 static const EMFolderViewEnable emfb_enable_map[];
 
@@ -202,7 +206,7 @@ emfb_init(GObject *o)
 
 	/* currently: just use a scrolledwindow for preview widget */
 	p->preview = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy((GtkScrolledWindow *)p->preview, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_policy((GtkScrolledWindow *)p->preview, GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
 	gtk_scrolled_window_set_shadow_type((GtkScrolledWindow *)p->preview, GTK_SHADOW_IN);
 	gtk_widget_show(p->preview);
 
@@ -212,15 +216,17 @@ emfb_init(GObject *o)
 	gtk_paned_add2((GtkPaned *)emfb->vpane, p->preview);
 	gtk_widget_show(p->preview);
 
-	g_signal_connect(emfb->view.list->tree, "key_press", G_CALLBACK(emfb_list_key_press), emfb);
+	g_signal_connect (((EMFolderView *) emfb)->list->tree, "key_press", G_CALLBACK(emfb_list_key_press), emfb);
+	g_signal_connect (((EMFolderView *) emfb)->list, "message_selected", G_CALLBACK (emfb_list_message_selected), emfb);
 }
 
 static void
 emfb_finalise(GObject *o)
 {
 	EMFolderBrowser *emfb = (EMFolderBrowser *)o;
-
-	g_free(emfb->priv);
+	
+	g_free (emfb->priv->select_uid);
+	g_free (emfb->priv);
 
 	((GObjectClass *)emfb_parent)->finalize(o);
 }
@@ -294,13 +300,17 @@ void em_folder_browser_show_preview(EMFolderBrowser *emfb, gboolean state)
 		gtk_widget_show (GTK_WIDGET (emfb->priv->preview));
 
 		if (emfb->view.list->cursor_uid)
-			message_list_select_uid(emfb->view.list, emfb->view.list->cursor_uid);
+			em_folder_view_set_message(&emfb->view, emfb->view.list->cursor_uid);
 
 		/* need to load/show the current message? */
 		/*do_message_selected (emfb);*/
 		/*set_cursor_pos (emfb, y);*/
 	} else {
-		em_format_format((EMFormat *)emfb->view.preview, NULL);
+		em_format_format((EMFormat *)emfb->view.preview, NULL, NULL, NULL);
+
+		g_free(emfb->view.displayed_uid);
+		emfb->view.displayed_uid = NULL;
+
 		gtk_widget_hide(emfb->priv->preview);
 		/*
 		mail_display_set_message (emfb->mail_display, NULL, NULL, NULL);
@@ -440,6 +450,18 @@ emfb_list_key_press(ETree *tree, int row, ETreePath path, int col, GdkEvent *ev,
 	return TRUE;
 }
 
+static void
+emfb_list_message_selected (MessageList *ml, const char *uid, EMFolderBrowser *emfb)
+{
+	EMFolderView *emfv = (EMFolderView *) emfb;
+	
+	if (emfv->folder == NULL)
+		return;
+	
+	camel_object_meta_set (emfv->folder, "evolution:selected_uid", uid);
+	camel_object_state_write (emfv->folder);
+}
+
 /* ********************************************************************** */
 
 static void
@@ -518,7 +540,7 @@ emfb_folder_expunge(BonoboUIComponent *uid, void *data, const char *path)
 {
 	EMFolderBrowser *emfb = data;
 	
-	em_utils_expunge_folder ((GtkWidget *) emfb, emfb->view.folder);
+	em_utils_expunge_folder(gtk_widget_get_toplevel((GtkWidget *)emfb), emfb->view.folder);
 }
 
 static void
@@ -573,18 +595,15 @@ emfb_empty_trash(BonoboUIComponent *uid, void *data, const char *path)
 {
 	EMFolderView *emfv = data;
 	
-	em_utils_empty_trash ((GtkWidget *) emfv);
-}
-
-static void
-emfb_forget_passwords(BonoboUIComponent *uid, void *data, const char *path)
-{
-	e_passwords_forget_passwords();
+	em_utils_empty_trash (gtk_widget_get_toplevel ((GtkWidget *) emfv));
 }
 
 static void
 emfb_mail_compose(BonoboUIComponent *uid, void *data, const char *path)
 {
+	if (!em_utils_check_user_can_send_mail((GtkWidget *)data))
+		return;
+
 	em_utils_compose_new_message ();
 }
 
@@ -598,11 +617,7 @@ static void
 emfb_mail_post(BonoboUIComponent *uid, void *data, const char *path)
 {
 	EMFolderView *emfv = data;
-	char *url;
-	
-	url = mail_tools_folder_to_url (emfv->folder);
-	em_utils_post_to_url (url);
-	g_free (url);
+	em_utils_post_to_folder (emfv->folder);
 }
 
 static void
@@ -659,7 +674,6 @@ static BonoboUIVerb emfb_verbs[] = {
 	/* ViewThreaded is a toggle */
 
 	BONOBO_UI_UNSAFE_VERB ("EmptyTrash", emfb_empty_trash),
-	BONOBO_UI_UNSAFE_VERB ("ForgetPasswords", emfb_forget_passwords),
 	BONOBO_UI_UNSAFE_VERB ("MailCompose", emfb_mail_compose),
 	BONOBO_UI_UNSAFE_VERB ("MailPost", emfb_mail_post),
 	BONOBO_UI_UNSAFE_VERB ("MailStop", emfb_mail_stop),
@@ -747,16 +761,24 @@ emfb_view_preview(BonoboUIComponent *uic, const char *path, Bonobo_UIComponent_E
 /* TODO: This should probably be handled by message-list, by storing/queueing
    up the select operation if its busy rebuilding the message-list */
 static void
-emfb_list_built(MessageList *ml, EMFolderBrowser *emfb)
+emfb_list_built (MessageList *ml, EMFolderBrowser *emfb)
 {
-	g_signal_handler_disconnect(ml, emfb->priv->list_built_id);
+	EMFolderView *emfv = (EMFolderView *) emfb;
+	
+	g_signal_handler_disconnect (ml, emfb->priv->list_built_id);
 	emfb->priv->list_built_id = 0;
 	
-	/* FIXME: if the 1st message in the list is unread, this will actually select the second unread msg */
-	/* FIXME: this usually happens "too soon" and so the toolbar/menu-items remain desensitised even tho with the message selected they should be sensitive */
-	if (((EMFolderView *)emfb)->list->cursor_uid == NULL)
-		message_list_select(((EMFolderView *)emfb)->list,
-				    MESSAGE_LIST_SELECT_NEXT, 0, CAMEL_MESSAGE_SEEN, TRUE);
+	if (emfv->list->cursor_uid == NULL) {
+		if (emfb->priv->select_uid) {
+			message_list_select_uid (ml, emfb->priv->select_uid);
+			g_free (emfb->priv->select_uid);
+			emfb->priv->select_uid = NULL;
+		} else {
+			/* NOTE: not all users want this, so we need a preference for it perhaps? see bug #52887 */
+			/* FIXME: if the 1st message in the list is unread, this will actually select the second unread msg */
+			/*message_list_select (ml, MESSAGE_LIST_SELECT_NEXT, 0, CAMEL_MESSAGE_SEEN, TRUE);*/
+		}
+	}
 }
 
 
@@ -868,28 +890,37 @@ emfb_create_view_instance (EMFolderBrowser *emfb, CamelFolder *folder, const cha
 static void
 emfb_set_folder(EMFolderView *emfv, CamelFolder *folder, const char *uri)
 {
+	EMFolderBrowser *emfb = (EMFolderBrowser *) emfv;
+
+	emfb_parent->set_folder(emfv, folder, uri);
+	
 	/* This is required since we get activated the first time
 	   before the folder is open and need to override the
 	   defaults */
 	if (folder) {
 		char *sstate;
 		
-		if ((sstate = camel_object_meta_get(folder, "evolution:show_preview")))
-			em_folder_browser_show_preview((EMFolderBrowser *)emfv, sstate[0] != '0');
+		if ((sstate = camel_object_meta_get (folder, "evolution:show_preview"))) {
+			em_folder_browser_show_preview ((EMFolderBrowser *) emfv, sstate[0] != '0');
+			g_free (sstate);
+		}
 		
-		if ((sstate = camel_object_meta_get(folder, "evolution:thread_list")))
-			message_list_set_threaded(emfv->list, sstate[0] != '0');
+		if ((sstate = camel_object_meta_get (folder, "evolution:thread_list"))) {
+			message_list_set_threaded (emfv->list, sstate[0] != '0');
+			g_free (sstate);
+		}
 		
-#if 0
-		if (emfv->list->cursor_uid == NULL && ((EMFolderBrowser *)emfv)->priv->list_built_id == 0)
-			((EMFolderBrowser *)emfv)->priv->list_built_id =
-				g_signal_connect(emfv->list, "message_list_built", G_CALLBACK(emfb_list_built), emfv);
-#endif
+		if ((sstate = camel_object_meta_get (folder, "evolution:selected_uid")))
+			emfb->priv->select_uid = sstate;
 		
-		emfb_create_view_instance ((EMFolderBrowser *) emfv, folder, uri);
+		if (emfv->list->cursor_uid == NULL && emfb->priv->list_built_id == 0)
+			emfb->priv->list_built_id =
+				g_signal_connect (emfv->list, "message_list_built", G_CALLBACK (emfb_list_built), emfv);
+		
+		/*emfb_create_view_instance ((EMFolderBrowser *) emfv, folder, uri);*/
+		if (emfv->uic)
+			emfb_create_view_menus((EMFolderBrowser *)emfv, emfv->uic);
 	}
-	
-	emfb_parent->set_folder(emfv, folder, uri);
 }
 
 /* TODO: All this mess should sit directly on MessageList, but it would
@@ -899,9 +930,19 @@ static void
 emfb_create_view_menus(EMFolderBrowser *emfb, BonoboUIComponent *uic)
 {
 	struct _EMFolderBrowserPrivate *p = emfb->priv;
-	
-	g_assert(p->view_instance == NULL);
-	g_assert(p->view_menus == NULL);
+
+	if (p->view_instance) {
+		g_object_unref(p->view_instance);
+		p->view_instance = NULL;
+	}
+
+	if (p->view_menus) {
+		g_object_unref(p->view_menus);
+		p->view_menus = NULL;
+	}
+
+	/*g_assert(p->view_instance == NULL);
+	  g_assert(p->view_menus == NULL);*/
 	
 	collection_init ();
 	
@@ -944,7 +985,7 @@ emfb_activate(EMFolderView *emfv, BonoboUIComponent *uic, int act)
 		gtk_paned_set_position((GtkPaned *)emfb->vpane, gconf_client_get_int (gconf, "/apps/evolution/mail/display/paned_size", NULL));
 		g_signal_handler_unblock(emfb->vpane, emfb->priv->vpane_resize_id);
 #endif
-	
+		
 		/* (Pre)view toggle */
 		if (emfv->folder
 		    && (sstate = camel_object_meta_get(emfv->folder, "evolution:show_preview"))) {
@@ -1006,6 +1047,9 @@ emfb_activate(EMFolderView *emfv, BonoboUIComponent *uic, int act)
 		if (p->view_instance) {
 			g_object_unref(p->view_instance);
 			p->view_instance = NULL;
+		}
+
+		if (p->view_menus) {
 			g_object_unref(p->view_menus);
 			p->view_menus = NULL;
 		}
