@@ -69,11 +69,15 @@
 #include <camel/camel-mime-message.h>
 #include <camel/camel-gpg-context.h>
 
+/* should this be in e-util rather than gal? */
+#include <gal/util/e-util.h>
+
 #include <e-util/e-msgport.h>
 #include <e-util/e-gui-utils.h>
 #include <e-util/e-dialog-utils.h>
 
 #if defined(HAVE_NSS)
+#include <camel/camel-smime-context.h>
 #include "certificate-viewer.h"
 #include "e-cert-db.h"
 #endif
@@ -105,6 +109,7 @@ struct _EMFormatHTMLDisplayPrivate {
 
 static int efhd_html_button_press_event (GtkWidget *widget, GdkEventButton *event, EMFormatHTMLDisplay *efh);
 static void efhd_html_link_clicked (GtkHTML *html, const char *url, EMFormatHTMLDisplay *efhd);
+static void efhd_html_on_url (GtkHTML *html, const char *url, EMFormatHTMLDisplay *efhd);
 
 struct _attach_puri {
 	EMFormatPURI puri;
@@ -123,24 +128,31 @@ static void efhd_iframe_created(GtkHTML *html, GtkHTML *iframe, EMFormatHTMLDisp
 /*static void efhd_url_requested(GtkHTML *html, const char *url, GtkHTMLStream *handle, EMFormatHTMLDisplay *efh);
   static gboolean efhd_object_requested(GtkHTML *html, GtkHTMLEmbedded *eb, EMFormatHTMLDisplay *efh);*/
 
-static void efhd_format_clone(EMFormat *, CamelMedium *, EMFormat *);
+static const EMFormatHandler *efhd_find_handler(EMFormat *emf, const char *mime_type);
+static void efhd_format_clone(EMFormat *, CamelFolder *folder, const char *, CamelMimeMessage *msg, EMFormat *);
+static void efhd_format_prefix(EMFormat *emf, CamelStream *stream);
 static void efhd_format_error(EMFormat *emf, CamelStream *stream, const char *txt);
 static void efhd_format_message(EMFormat *, CamelStream *, CamelMedium *);
 static void efhd_format_source(EMFormat *, CamelStream *, CamelMimePart *);
 static void efhd_format_attachment(EMFormat *, CamelStream *, CamelMimePart *, const char *, const EMFormatHandler *);
 static void efhd_complete(EMFormat *);
 
+static gboolean efhd_bonobo_object(EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObject *pobject);
+static gboolean efhd_use_component(const char *mime_type);
+
 static void efhd_builtin_init(EMFormatHTMLDisplayClass *efhc);
 
 enum {
 	EFHD_LINK_CLICKED,
 	EFHD_POPUP_EVENT,
+	EFHD_ON_URL,
 	EFHD_LAST_SIGNAL,
 };
 
 static guint efhd_signals[EFHD_LAST_SIGNAL] = { 0 };
 
-
+/* EMFormatHandler's for bonobo objects */
+static GHashTable *efhd_bonobo_handlers;
 static EMFormatHTMLClass *efhd_parent;
 
 static void
@@ -254,7 +266,9 @@ efhd_bool_accumulator(GSignalInvocationHint *ihint, GValue *out, const GValue *i
 static void
 efhd_class_init(GObjectClass *klass)
 {
+	((EMFormatClass *)klass)->find_handler = efhd_find_handler;
 	((EMFormatClass *)klass)->format_clone = efhd_format_clone;
+	((EMFormatClass *)klass)->format_prefix = efhd_format_prefix;
 	((EMFormatClass *)klass)->format_error = efhd_format_error;
 	((EMFormatClass *)klass)->format_message = efhd_format_message;
 	((EMFormatClass *)klass)->format_source = efhd_format_source;
@@ -283,6 +297,16 @@ efhd_class_init(GObjectClass *klass)
 			     GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE,
 			     G_TYPE_POINTER, G_TYPE_POINTER);
 
+	efhd_signals[EFHD_ON_URL] = 
+		g_signal_new("on_url",
+			     G_TYPE_FROM_CLASS(klass),
+			     G_SIGNAL_RUN_LAST,
+			     G_STRUCT_OFFSET(EMFormatHTMLDisplayClass, on_url),
+			     NULL, NULL,
+			     gtk_marshal_VOID__STRING,
+			     G_TYPE_NONE, 1,
+			     G_TYPE_STRING);
+
 	efhd_builtin_init((EMFormatHTMLDisplayClass *)klass);
 }
 
@@ -302,6 +326,8 @@ em_format_html_display_get_type(void)
 		};
 		efhd_parent = g_type_class_ref(em_format_html_get_type());
 		type = g_type_register_static(em_format_html_get_type(), "EMFormatHTMLDisplay", &info, 0);
+
+		efhd_bonobo_handlers = g_hash_table_new(g_str_hash, g_str_equal);
 	}
 
 	return type;
@@ -315,6 +341,7 @@ EMFormatHTMLDisplay *em_format_html_display_new(void)
 
 	g_signal_connect(efhd->formathtml.html, "iframe_created", G_CALLBACK(efhd_iframe_created), efhd);
 	g_signal_connect(efhd->formathtml.html, "link_clicked", G_CALLBACK(efhd_html_link_clicked), efhd);
+	g_signal_connect(efhd->formathtml.html, "on_url", G_CALLBACK(efhd_html_on_url), efhd);
 	g_signal_connect(efhd->formathtml.html, "button_press_event", G_CALLBACK(efhd_html_button_press_event), efhd);
 
 	return efhd;
@@ -361,7 +388,7 @@ em_format_html_display_set_search(EMFormatHTMLDisplay *efhd, int type, GSList *s
 	}
 
 	d(printf("redrawing with search\n"));
-	em_format_format_clone((EMFormat *)efhd, ((EMFormat *)efhd)->message, (EMFormat *)efhd);
+	em_format_redraw((EMFormat *)efhd);
 }
 
 static void
@@ -543,26 +570,27 @@ static int
 efhd_html_button_press_event (GtkWidget *widget, GdkEventButton *event, EMFormatHTMLDisplay *efhd)
 {
 	HTMLEngine *e;
-	HTMLPoint *point;
+	HTMLObject *obj;
 	const char *url;
 	gboolean res = FALSE;
+	gint offset;
 
 	if (event->button != 3)
 		return FALSE;
 
 	e = ((GtkHTML *)widget)->engine;
-	point = html_engine_get_point_at(e, event->x, event->y, FALSE);			
-	if (point == NULL)
+	obj = html_engine_get_object_at(e, event->x, event->y, &offset, FALSE);
+	if (obj == NULL)
 		return FALSE;
 
 	d(printf("popup button pressed\n"));
 
-	if ( (url = html_object_get_src(point->object)) != NULL
-	     || (url = html_object_get_url(point->object, 0)) != NULL) {
+	if ( (url = html_object_get_src(obj)) != NULL
+	     || (url = html_object_get_url(obj, offset)) != NULL) {
 		EMFormatPURI *puri;
 		char *uri;
 
-		uri = gtk_html_get_url_object_relative((GtkHTML *)widget, point->object, url);
+		uri = gtk_html_get_url_object_relative((GtkHTML *)widget, obj, url);
 		puri = em_format_find_puri((EMFormat *)efhd, uri);
 
 		d(printf("poup event, uri = '%s' part = '%p'\n", uri, puri?puri->part:NULL));
@@ -570,8 +598,6 @@ efhd_html_button_press_event (GtkWidget *widget, GdkEventButton *event, EMFormat
 		g_signal_emit((GtkObject *)efhd, efhd_signals[EFHD_POPUP_EVENT], 0, event, uri, puri?puri->part:NULL, &res);
 		g_free(uri);
 	}
-	
-	html_point_destroy(point);
 
 	return res;
 }
@@ -583,6 +609,13 @@ efhd_html_link_clicked (GtkHTML *html, const char *url, EMFormatHTMLDisplay *efh
 	g_signal_emit((GObject *)efhd, efhd_signals[EFHD_LINK_CLICKED], 0, url);
 }
 
+static void
+efhd_html_on_url (GtkHTML *html, const char *url, EMFormatHTMLDisplay *efhd)
+{
+	d(printf("on_url event '%s'\n", url));
+	g_signal_emit((GObject *)efhd, efhd_signals[EFHD_ON_URL], 0, url);
+}
+
 static void
 efhd_complete(EMFormat *emf)
 {
@@ -609,7 +642,7 @@ static const struct {
 static const struct {
 	const char *icon, *shortdesc, *description;
 } smime_encrypt_table[4] = {
-	{ NULL, N_("Unencrypted"), N_("This message is not encrypted.  It's content may be viewed in transit across The Internet.") },
+	{ NULL, N_("Unencrypted"), N_("This message is not encrypted.  Its content may be viewed in transit across the Internet.") },
 	{ "pgp-signature-ok.png", N_("Encrypted, weak"), N_("This message is encrypted, but with a weak encryption algorithm.  It would be difficult, but not impossible for an outsider to view the content of this message in a practical amount of time.") },
 	{ "pgp-signature-ok.png", N_("Encrypted"), N_("This message is encrypted.  It would be difficult for an outsider to view the content of this message.") },
 	{ "pgp-signature-ok.png", N_("Encrypted, strong"), N_("This message is encrypted, with a strong encryption algorithm.  It would be very difficult for an outsider to view the content of this message in a practical amount of time.") },
@@ -807,9 +840,9 @@ static void
 efhd_output_secure(EMFormat *emf, CamelStream *stream, CamelMimePart *part, CamelCipherValidity *valid)
 {
 	CamelCipherValidity *save = ((EMFormatHTML *)emf)->valid_parent;
+	int len;
 
 	/* Note: this same logic is in efh_output_secure */
-
 	if (((EMFormatHTML *)emf)->valid == NULL) {
 		((EMFormatHTML *)emf)->valid = valid;
 	} else {
@@ -818,7 +851,12 @@ efhd_output_secure(EMFormat *emf, CamelStream *stream, CamelMimePart *part, Came
 	}
 
 	((EMFormatHTML *)emf)->valid_parent = valid;
+
+	len = emf->part_id->len;
+	g_string_append_printf(emf->part_id, ".signed");
 	em_format_part(emf, stream, part);
+	g_string_truncate(emf->part_id, len);
+
 	((EMFormatHTML *)emf)->valid_parent = save;
 
 	if (((EMFormatHTML *)emf)->valid == valid
@@ -829,7 +867,8 @@ efhd_output_secure(EMFormat *emf, CamelStream *stream, CamelMimePart *part, Came
 
 		camel_stream_printf(stream, "<table border=0 width=\"100%%\" cellpadding=3 cellspacing=0 bgcolor=%s><tr>",
 				    valid->sign.status == CAMEL_CIPHER_VALIDITY_SIGN_GOOD?"#88bb88":"#bb8888");
-		classid = g_strdup_printf("smime:///em-format-html/%p/icon/signed", part);
+
+		classid = g_strdup_printf("smime:///em-format-html/%s/icon/signed", emf->part_id->str);
 		pobj = (struct _smime_pobject *)em_format_html_add_pobject((EMFormatHTML *)emf, sizeof(*pobj), classid, part, efhd_xpkcs7mime_button);
 		pobj->valid = camel_cipher_validity_clone(valid);
 		pobj->object.free = efhd_xpkcs7mime_free;
@@ -938,9 +977,108 @@ efhd_builtin_init(EMFormatHTMLDisplayClass *efhc)
 
 /* ********************************************************************** */
 
-static void efhd_format_clone(EMFormat *emf, CamelMedium *part, EMFormat *src)
+static void
+efhd_bonobo_unknown(EMFormat *emf, CamelStream *stream, CamelMimePart *part, const EMFormatHandler *info)
 {
-	((EMFormatClass *)efhd_parent)->format_clone(emf, part, src);
+	char *classid;
+
+	classid = g_strdup_printf("bonobo-unknown:///em-format-html-display/%s", emf->part_id->str);
+	em_format_html_add_pobject((EMFormatHTML *)emf, sizeof(EMFormatHTMLPObject), classid, part, efhd_bonobo_object);
+	camel_stream_printf(stream, "<object classid=\"%s\" type=\"%s\">\n", classid, info->mime_type);
+	g_free(classid);
+}
+
+/* ********************************************************************** */
+
+static const EMFormatHandler *efhd_find_handler(EMFormat *emf, const char *mime_type)
+{
+	const EMFormatHandler *handle;
+
+	if (efhd_use_component(mime_type)) {
+		if ((handle = g_hash_table_lookup(efhd_bonobo_handlers, mime_type)) == NULL) {
+			EMFormatHandler *h = g_malloc0(sizeof(*h));
+
+			h->mime_type = g_strdup(mime_type);
+			h->handler = efhd_bonobo_unknown;
+			g_hash_table_insert(efhd_bonobo_handlers, h->mime_type, h);
+
+			handle = h;
+		}
+	} else {
+		handle = ((EMFormatClass *)efhd_parent)->find_handler(emf, mime_type);
+	}
+
+	return handle;
+}
+
+static void efhd_format_clone(EMFormat *emf, CamelFolder *folder, const char *uid, CamelMimeMessage *msg, EMFormat *src)
+{
+	((EMFormatClass *)efhd_parent)->format_clone(emf, folder, uid, msg, src);
+}
+
+static void
+efhd_write_image(EMFormat *emf, CamelStream *stream, EMFormatPURI *puri)
+{
+	CamelDataWrapper *dw = camel_medium_get_content_object((CamelMedium *)puri->part);
+
+	/* TODO: identical to efh_write_image */
+	d(printf("writing image '%s'\n", puri->uri?puri->uri:puri->cid));
+	camel_data_wrapper_decode_to_stream(dw, stream);
+	camel_stream_close(stream);
+}
+
+static void efhd_format_prefix(EMFormat *emf, CamelStream *stream)
+{
+	const char *flag, *comp, *due;
+	CamelMimePart *iconpart;
+	time_t date;
+	char due_date[128];
+	struct tm due_tm;
+
+	if (emf->folder == NULL || emf->uid == NULL
+	    || (flag = camel_folder_get_message_user_tag(emf->folder, emf->uid, "follow-up")) == NULL
+	    || flag[0] == 0)
+		return;
+
+	/* header displayed for message-flags in mail display */
+	camel_stream_printf(stream, "<table border=1 width=\"100%%\" cellspacing=2 cellpadding=2><tr>");
+
+	comp = camel_folder_get_message_user_tag(emf->folder, emf->uid, "completed-on");
+	iconpart = em_format_html_file_part((EMFormatHTML *)emf, "image/png", EVOLUTION_ICONSDIR,
+					    comp&&comp[0]?"flag-for-followup-done-16.png":"flag-for-followup-16.png");
+	if (iconpart) {
+		char *classid;
+
+		classid = g_strdup_printf("icon:///em-format-html-display/%s/%s", emf->part_id->str, comp&&comp[0]?"comp":"uncomp");
+		camel_stream_printf(stream, "<td align=\"left\"><img src=\"%s\"></td>", classid);
+		(void)em_format_add_puri(emf, sizeof(EMFormatPURI), classid, iconpart, efhd_write_image);
+		g_free(classid);
+		camel_object_unref(iconpart);
+	}
+
+	camel_stream_printf(stream, "<td align=\"left\" width=\"100%%\">");
+
+	if (comp && comp[0]) {
+		date = camel_header_decode_date(comp, NULL);
+		localtime_r(&date, &due_tm);
+		e_utf8_strftime_fix_am_pm(due_date, sizeof (due_date), _("Completed on %B %d, %Y, %l:%M %p"), &due_tm);
+		camel_stream_printf(stream, "%s, %s", flag, due_date);
+	} else if ((due = camel_folder_get_message_user_tag(emf->folder, emf->uid, "due-by")) != NULL && due[0]) {
+		time_t now;
+
+		date = camel_header_decode_date(due, NULL);
+		now = time(NULL);
+		if (now > date)
+			camel_stream_printf(stream, "<b>%s</b>&nbsp;", _("Overdue:"));
+		
+		localtime_r(&date, &due_tm);
+		e_utf8_strftime_fix_am_pm(due_date, sizeof (due_date), _("by %B %d, %Y, %l:%M %p"), &due_tm);
+		camel_stream_printf(stream, "%s %s", flag, due_date);
+	} else {
+		camel_stream_printf(stream, "%s", flag);
+	}
+
+	camel_stream_printf(stream, "</td></tr></table>");
 }
 
 /* TODO: if these aren't going to do anything should remove */
@@ -970,7 +1108,7 @@ efhd_attachment_show(GtkWidget *w, struct _attach_puri *info)
 	info->shown = ~info->shown;
 	em_format_set_inline(info->puri.format, info->puri.part, info->shown);
 	/* FIXME: do this in an idle handler */
-	em_format_format_clone(info->puri.format, info->puri.format->message, info->puri.format);
+	em_format_redraw(info->puri.format);
 #if 0
 	/* FIXME: track shown state in parent */
 
@@ -1186,14 +1324,24 @@ efhd_attachment_button(EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObje
 	simple_type = camel_content_type_simple (((CamelDataWrapper *)pobject->part)->mime_type);
 	camel_strdown(simple_type);
 
-	/* cache? */
 	/* FIXME: offline parts, just get icon */
 	if (camel_content_type_is (((CamelDataWrapper *)pobject->part)->mime_type, "image", "*")) {
 		EMFormatHTMLJob *job;
+		GdkPixbuf *mini;
+		char *key;
 
-		job = em_format_html_job_new(efh, efhd_write_icon_job, pobject);
-		job->stream = (CamelStream *)em_icon_stream_new((GtkImage *)w);
-		em_format_html_job_queue(efh, job);
+		key = pobject->classid;
+		mini = em_icon_stream_get_image(key);
+		if (mini) {
+			d(printf("got image from cache '%s'\n", key));
+			gtk_image_set_from_pixbuf((GtkImage *)w, mini);
+			g_object_unref(mini);
+		} else {
+			d(printf("need to create icon image '%s'\n", key));
+			job = em_format_html_job_new(efh, efhd_write_icon_job, pobject);
+			job->stream = (CamelStream *)em_icon_stream_new((GtkImage *)w, key);
+			em_format_html_job_queue(efh, job);
+		}
 	} else {
 		GdkPixbuf *pixbuf = e_icon_for_mime_type(simple_type, 24);
 		GdkPixbuf *mini = gdk_pixbuf_scale_simple(pixbuf, 24, 24, GDK_INTERP_BILINEAR);
@@ -1294,7 +1442,7 @@ efhd_bonobo_object(EMFormatHTML *efh, GtkHTMLEmbedded *eb, EMFormatHTMLPObject *
 	}
 	
 	persist = (Bonobo_PersistStream)Bonobo_Unknown_queryInterface(bonobo_widget_get_objref((BonoboWidget *)embedded),
-								      "IDL:Bonobo/PersistStream:1.0", NULL);
+								      "IDL:Bonobo/PersistStream:1.0", &ev);
 	if (persist == CORBA_OBJECT_NIL) {
 		gtk_object_sink((GtkObject *)embedded);
 		CORBA_exception_free(&ev);				
@@ -1394,7 +1542,7 @@ efhd_format_attachment(EMFormat *emf, CamelStream *stream, CamelMimePart *part,
 	char *classid, *text, *html;
 	struct _attach_puri *info;
 
-	classid = g_strdup_printf("attachment-%p", part);
+	classid = g_strdup_printf("attachment%s", emf->part_id->str);
 	info = (struct _attach_puri *)em_format_add_puri(emf, sizeof(*info), classid, part, efhd_attachment_frame);
 	em_format_html_add_pobject((EMFormatHTML *)emf, sizeof(EMFormatHTMLPObject), classid, part, efhd_attachment_button);
 	info->handle = handle;
@@ -1429,11 +1577,9 @@ efhd_format_attachment(EMFormat *emf, CamelStream *stream, CamelMimePart *part,
 			handle->handler(emf, stream, part, handle);
 		/*camel_stream_printf(stream, "<iframe src=\"%s\" marginheight=0 marginwidth=0>%s</iframe>\n", classid, _("Attachment content could not be loaded"));*/
 	} else if (efhd_use_component(mime_type)) {
-		static int partid;
-
 		g_free(classid); /* messy */
 
-		classid = g_strdup_printf("bonobo-unknown:///em-formath-html-display/%p/%d", part, partid++);
+		classid = g_strdup_printf("bonobo-unknown:///em-format-html-display/%s", emf->part_id->str);
 		em_format_html_add_pobject((EMFormatHTML *)emf, sizeof(EMFormatHTMLPObject), classid, part, efhd_bonobo_object);
 		camel_stream_printf(stream, "<object classid=\"%s\" type=\"%s\">\n", classid, mime_type);
 	}
