@@ -76,6 +76,7 @@ typedef struct _MailSession {
 	/* spamassassin filter options */
 	gboolean sa_local_only;
 	gboolean sa_use_daemon;
+	int sa_daemon_port;
 } MailSession;
 
 typedef struct _MailSessionClass {
@@ -85,7 +86,7 @@ typedef struct _MailSessionClass {
 
 static CamelSessionClass *ms_parent_class;
 
-static char *get_password(CamelSession *session, const char *prompt, gboolean reprompt, gboolean secret, CamelService *service, const char *item, CamelException *ex);
+static char *get_password(CamelSession *session, const char *prompt, guint32 flags, CamelService *service, const char *item, CamelException *ex);
 static void forget_password(CamelSession *session, CamelService *service, const char *item, CamelException *ex);
 static gboolean alert_user(CamelSession *session, CamelSessionAlertType type, const char *prompt, gboolean cancel);
 static CamelFilterDriver *get_filter_driver(CamelSession *session, const char *type, CamelException *ex);
@@ -174,8 +175,7 @@ struct _pass_msg {
 	
 	CamelSession *session;
 	const char *prompt;
-	gboolean reprompt;
-	gboolean secret;
+	guint32 flags;
 	CamelService *service;
 	const char *item;
 	CamelException *ex;
@@ -283,20 +283,23 @@ request_password (struct _pass_msg *m)
 	gtk_container_set_border_width ((GtkContainer *) password_dialog, 6);
 	
 	m->entry = gtk_entry_new ();
-	gtk_entry_set_visibility ((GtkEntry *) m->entry, !m->secret);
+	gtk_entry_set_visibility ((GtkEntry *) m->entry, !(m->flags & CAMEL_SESSION_PASSWORD_SECRET));
 	g_signal_connect (m->entry, "activate", G_CALLBACK (pass_activate), password_dialog);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (password_dialog)->vbox), m->entry, TRUE, FALSE, 3);
 	gtk_widget_show (m->entry);
 	
-	if (m->reprompt && m->result) {
+	if ((m->flags & CAMEL_SESSION_PASSWORD_REPROMPT) && m->result) {
 		gtk_entry_set_text ((GtkEntry *) m->entry, m->result);
 		g_free (m->result);
 		m->result = NULL;
 	}
-	
-	if (m->service_url == NULL || m->service != NULL) {
-		m->check = gtk_check_button_new_with_mnemonic (m->service_url ? _("_Remember this password") :
-							       _("_Remember this password for the remainder of this session"));
+
+	/* static password, shouldn't be remembered between sessions,
+	   but will be remembered within the session beyond our control */
+	if ((m->service_url == NULL || m->service != NULL)
+	    && (m->flags & CAMEL_SESSION_PASSWORD_STATIC) == 0) {
+		m->check = gtk_check_button_new_with_mnemonic (m->service_url ? _("_Remember this password")
+							       : _("_Remember this password for the remainder of this session"));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m->check),
 					      m->config_service ? m->config_service->save_passwd : FALSE);
 		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (password_dialog)->vbox), m->check, TRUE, FALSE, 3);
@@ -326,7 +329,7 @@ do_get_pass(struct _mail_msg *mm)
 			m->result = g_strdup(account->source->url);
 	} else if (m->key) {
 		m->result = e_passwords_get_password ("Mail", m->key);
-		if (m->result == NULL || m->reprompt) {
+		if (m->result == NULL || (m->flags & CAMEL_SESSION_PASSWORD_REPROMPT)) {
 			if (mail_session->interactive) {
 				request_password(m);
 				return;
@@ -354,7 +357,7 @@ static struct _mail_msg_op get_pass_op = {
 };
 
 static char *
-get_password (CamelSession *session, const char *prompt, gboolean reprompt, gboolean secret,
+get_password (CamelSession *session, const char *prompt, guint32 flags,
 	      CamelService *service, const char *item, CamelException *ex)
 {
 	struct _pass_msg *m, *r;
@@ -369,8 +372,7 @@ get_password (CamelSession *session, const char *prompt, gboolean reprompt, gboo
 	m->ismain = pthread_self() == mail_gui_thread;
 	m->session = session;
 	m->prompt = prompt;
-	m->reprompt = reprompt;
-	m->secret = secret;
+	m->flags = flags;
 	m->service = service;
 	m->item = item;
 	m->ex = ex;
@@ -644,6 +646,9 @@ main_get_filter_driver (CamelSession *session, const char *type, CamelException
 		fsearch = g_string_new ("");
 		faction = g_string_new ("");
 
+		if (!strcmp (type, FILTER_SOURCE_DEMAND))
+			type = FILTER_SOURCE_INCOMING;
+
 		/* add the user-defined rules next */
 		while ((rule = rule_context_next_rule (fc, rule, type))) {
 			g_string_truncate (fsearch, 0);
@@ -784,6 +789,18 @@ mail_session_set_sa_use_daemon (gboolean value)
 	MAIL_SESSION (session)->sa_use_daemon = value;
 }
 
+int
+mail_session_get_sa_daemon_port ()
+{
+	return MAIL_SESSION (session)->sa_daemon_port;
+}
+
+void
+mail_session_set_sa_daemon_port (int value)
+{
+	MAIL_SESSION (session)->sa_daemon_port = value;
+}
+
 static void
 mail_session_check_junk_notify (GConfClient *gconf, guint id, GConfEntry *entry, CamelSession *session)
 {
@@ -797,10 +814,15 @@ mail_session_check_junk_notify (GConfClient *gconf, guint id, GConfEntry *entry,
 		key ++;
 		if (!strcmp (key, "check_incoming"))
 			camel_session_set_check_junk (session, gconf_value_get_bool (gconf_entry_get_value (entry)));
+		else if (!strcmp (key, "check_incoming_imap"))
+			camel_session_set_check_junk_for_imap (session, gconf_value_get_bool (gconf_entry_get_value (entry)));
 		else if (!strcmp (key, "local_only"))
 			mail_session_set_sa_local_only (gconf_value_get_bool (gconf_entry_get_value (entry)));
 		else if (!strcmp (key, "use_daemon"))
 			mail_session_set_sa_use_daemon (gconf_value_get_bool (gconf_entry_get_value (entry)));
+		else if (!strcmp (key, "daemon_port"))
+			mail_session_set_sa_daemon_port (gconf_value_get_int (gconf_entry_get_value (entry)));
+
 	}
 }
 
@@ -821,6 +843,8 @@ mail_session_init (const char *base_directory)
 	gconf = mail_config_get_gconf_client ();
 	gconf_client_add_dir (gconf, "/apps/evolution/mail/junk", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 	camel_session_set_check_junk (session, gconf_client_get_bool (gconf, "/apps/evolution/mail/junk/check_incoming", NULL));
+	camel_session_set_check_junk_for_imap (session, gconf_client_get_bool (gconf, "/apps/evolution/mail/junk/check_incoming_imap", NULL));
+	mail_session_set_sa_daemon_port (gconf_client_get_int (gconf, "/apps/evolution/mail/junk/sa/daemon_port", NULL));
 	session_check_junk_notify_id = gconf_client_notify_add (gconf, "/apps/evolution/mail/junk",
 								(GConfClientNotifyFunc) mail_session_check_junk_notify,
 								session, NULL, NULL);
