@@ -51,7 +51,7 @@
 #define d(x) /*(printf("%s(%d):%s: ",  __FILE__, __LINE__, __PRETTY_FUNCTION__), (x))*/
 
 static VfolderContext *context;	/* context remains open all time */
-static CamelStore *vfolder_store; /* the 1 static vfolder store */
+CamelStore *vfolder_store; /* the 1 static vfolder store */
 
 /* lock for accessing shared resources (below) */
 static pthread_mutex_t vfolder_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -168,6 +168,7 @@ static struct _mail_msg_op vfolder_setup_op = {
 	vfolder_setup_free,
 };
 
+/* sources_uri should be camel uri's */
 static int
 vfolder_setup(CamelFolder *folder, const char *query, GList *sources_uri, GList *sources_folder)
 {
@@ -261,6 +262,8 @@ static struct _mail_msg_op vfolder_adduri_op = {
 	vfolder_adduri_free,
 };
 
+
+/* uri should be a camel uri */
 static int
 vfolder_adduri(const char *uri, GList *folders, int remove)
 {
@@ -292,6 +295,7 @@ my_list_find(GList *l, const char *uri, GCompareFunc cmp)
 	return l;
 }
 
+/* uri is a camel uri */
 static int
 uri_is_ignore(const char *uri, GCompareFunc uri_cmp)
 {
@@ -307,6 +311,9 @@ uri_is_ignore(const char *uri, GCompareFunc uri_cmp)
 		|| (default_sent_folder_uri && uri_cmp(default_sent_folder_uri, uri))
 		|| (default_drafts_folder_uri && uri_cmp(default_drafts_folder_uri, uri));
 	
+	if (found)
+		return found;
+
 	accounts = mail_config_get_accounts ();
 	iter = e_list_get_iterator ((EList *) accounts);
 	while (e_iterator_is_valid (iter)) {
@@ -331,7 +338,7 @@ uri_is_ignore(const char *uri, GCompareFunc uri_cmp)
 
 /* called when a new uri becomes (un)available */
 void
-mail_vfolder_add_uri(CamelStore *store, const char *uri, int remove)
+mail_vfolder_add_uri(CamelStore *store, const char *curi, int remove)
 {
 	FilterRule *rule;
 	const char *source;
@@ -340,20 +347,26 @@ mail_vfolder_add_uri(CamelStore *store, const char *uri, int remove)
 	int remote = (((CamelService *)store)->provider->flags & CAMEL_PROVIDER_IS_REMOTE) != 0;
 	GCompareFunc uri_cmp = CAMEL_STORE_CLASS(CAMEL_OBJECT_GET_CLASS(store))->compare_folder_name;
 	int is_ignore;
+	char *uri;
 
-	if (CAMEL_IS_VEE_STORE(store) || !strncmp(uri, "vtrash:", 7) || !strncmp(uri, "vjunk:", 6) || context == NULL)
+	uri = em_uri_from_camel(curi);
+	if (!strncmp(curi, "vtrash:", 7) || !strncmp(curi, "vjunk:", 6) || context == NULL) {
+		g_free(uri);
 		return;
+	}
 
 	g_assert(pthread_self() == mail_gui_thread);
 
-	is_ignore = uri_is_ignore(uri, uri_cmp);
+	is_ignore = uri_is_ignore(curi, uri_cmp);
 
 	LOCK();
 
 	d(printf("%s uri to check: %s\n", remove?"Removing":"Adding", uri));
 
 	/* maintain the source folders lists for changed rules later on */
-	if (remove) {
+	if (CAMEL_IS_VEE_STORE(store)) {
+		is_ignore = TRUE;
+	} else if (remove) {
 		if (remote) {
 			if ((link = my_list_find(source_folders_remote, (void *)uri, uri_cmp)) != NULL) {
 				g_free(link->data);
@@ -395,9 +408,15 @@ mail_vfolder_add_uri(CamelStore *store, const char *uri, int remove)
 		
 		/* we check using the store uri_cmp since its more accurate */
 		source = NULL;
-		while (!found && (source = vfolder_rule_next_source((VfolderRule *)rule, source)))
-			found = uri_cmp(uri, source);
-		
+		while (!found && (source = vfolder_rule_next_source((VfolderRule *)rule, source))) {
+			char *esource;
+
+			esource = em_uri_from_camel(source);
+			found = uri_cmp(uri, esource);
+			d(printf(found?" '%s' == '%s'?\n":" '%s' != '%s'\n", uri, esource));
+			g_free(esource);
+		}
+
 		if (found) {
 			vf = g_hash_table_lookup(vfolder_hash, rule->name);
 			g_assert(vf);
@@ -409,22 +428,28 @@ mail_vfolder_add_uri(CamelStore *store, const char *uri, int remove)
 	UNLOCK();
 	
 	if (folders != NULL)
-		vfolder_adduri(uri, folders, remove);
+		vfolder_adduri(curi, folders, remove);
+
+	g_free(uri);
 }
 
 /* called when a uri is deleted from a store */
 void
-mail_vfolder_delete_uri(CamelStore *store, const char *uri)
+mail_vfolder_delete_uri(CamelStore *store, const char *curi)
 {
 	GCompareFunc uri_cmp = CAMEL_STORE_CLASS(CAMEL_OBJECT_GET_CLASS(store))->compare_folder_name;
 	FilterRule *rule;
 	const char *source;
 	CamelVeeFolder *vf;
 	GString *changed;
+	char *uri;
+	GList *link;
 
-	if (context == NULL || !strncmp(uri, "vtrash:", 7) || !strncmp(uri, "vjunk:", 6))
+	if (context == NULL || !strncmp(curi, "vtrash:", 7) || !strncmp(curi, "vjunk:", 6))
 		return;
-	
+
+	uri = em_uri_from_camel(curi);
+
 	d(printf ("Deleting uri to check: %s\n", uri));
 	
 	g_assert (pthread_self() == mail_gui_thread);
@@ -452,6 +477,16 @@ mail_vfolder_delete_uri(CamelStore *store, const char *uri)
 			}
 		}
 	}
+
+	if ((link = my_list_find(source_folders_remote, (void *)uri, uri_cmp)) != NULL) {
+		g_free(link->data);
+		source_folders_remote = g_list_remove_link(source_folders_remote, link);
+	}
+
+	if ((link = my_list_find(source_folders_local, (void *)uri, uri_cmp)) != NULL) {
+		g_free(link->data);
+		source_folders_local = g_list_remove_link(source_folders_local, link);
+	}
 	
 	UNLOCK();
 	
@@ -474,25 +509,31 @@ mail_vfolder_delete_uri(CamelStore *store, const char *uri)
 	}
 	
 	g_string_free (changed, TRUE);
+
+	g_free(uri);
 }
 
 /* called when a uri is renamed in a store */
 void
-mail_vfolder_rename_uri(CamelStore *store, const char *from, const char *to)
+mail_vfolder_rename_uri(CamelStore *store, const char *cfrom, const char *cto)
 {
 	GCompareFunc uri_cmp = CAMEL_STORE_CLASS(CAMEL_OBJECT_GET_CLASS(store))->compare_folder_name;
 	FilterRule *rule;
 	const char *source;
 	CamelVeeFolder *vf;
 	int changed = 0;
+	char *from, *to;
 
 	d(printf("vfolder rename uri: %s to %s\n", from, to));
 
-	if (context == NULL || !strncmp(from, "vtrash:", 7) || !strncmp(to, "vtrash:", 7) || !strncmp(from, "vjunk:", 6) || !strncmp(to, "vjunk:", 6))
+	if (context == NULL || !strncmp(cfrom, "vtrash:", 7) || !strncmp(cto, "vtrash:", 7) || !strncmp(cfrom, "vjunk:", 6) || !strncmp(cto, "vjunk:", 6))
 		return;
 
 	g_assert(pthread_self() == mail_gui_thread);
 
+	from = em_uri_from_camel(cfrom);
+	to = em_uri_from_camel(cto);
+
 	LOCK();
 
 	/* see if any rules directly reference this removed uri */
@@ -527,6 +568,9 @@ mail_vfolder_rename_uri(CamelStore *store, const char *from, const char *to)
 		rule_context_save((RuleContext *)context, user);
 		g_free(user);
 	}
+
+	g_free(from);
+	g_free(to);
 }
 
 /* ********************************************************************** */
@@ -571,12 +615,19 @@ rule_changed(FilterRule *rule, CamelFolder *folder)
 	/* find any (currently available) folders, and add them to the ones to open */
 	sourceuri = NULL;
 	while ( (sourceuri = vfolder_rule_next_source((VfolderRule *)rule, sourceuri)) ) {
-		if (mail_note_get_folder_from_uri(sourceuri, &newfolder)) {
+		char *curi = em_uri_to_camel(sourceuri);
+
+		d(printf(" adding source '%s' '%s'\n", sourceuri, curi));
+		if (mail_note_get_folder_from_uri(curi, &newfolder)) {
 			if (newfolder)
 				sources_folder = g_list_append(sources_folder, newfolder);
 			else
 				sources_uri = g_list_append(sources_uri, g_strdup(sourceuri));
+		} else {
+			d(printf("  no i'm not - this folder doesn't exist anywhere\n"));
 		}
+
+		g_free(curi);
 	}
 
 	/* check the remote/local uri lists for any other uri's that should be looked at */
@@ -591,14 +642,17 @@ rule_changed(FilterRule *rule, CamelFolder *folder)
 				l = NULL;
 
 			while (l) {
-				if (mail_note_get_folder_from_uri(l->data, &newfolder)) {
+				char *curi = em_uri_to_camel(l->data);
+
+				if (mail_note_get_folder_from_uri(curi, &newfolder)) {
 					if (newfolder)
 						sources_folder = g_list_append(sources_folder, newfolder);
 					else
-						sources_uri = g_list_append(sources_uri, g_strdup(l->data));
+						sources_uri = g_list_append(sources_uri, g_strdup(curi));
 				} else {
 					d(printf("  -> No such folder?\n"));
 				}
+				g_free(curi);
 				l = l->next;
 			}
 		}
@@ -710,7 +764,6 @@ store_folder_deleted(CamelObject *o, void *event_data, void *data)
 static void
 store_folder_renamed(CamelObject *o, void *event_data, void *data)
 {
-	CamelStore *store = (CamelStore *)o;
 	CamelRenameInfo *info = event_data;
 	FilterRule *rule;
 	char *user;
@@ -756,7 +809,7 @@ vfolder_load_storage(void)
 	vfolder_hash = g_hash_table_new(g_str_hash, g_str_equal);
 
 	/* first, create the vfolder store, and set it up */
-	storeuri = g_strdup_printf("vfolder:%s/vfolder", mail_component_peek_base_directory (mail_component_peek ()));
+	storeuri = g_strdup_printf("vfolder:%s/mail/vfolder", mail_component_peek_base_directory (mail_component_peek ()));
 	vfolder_store = camel_session_get_store(session, storeuri, NULL);
 	if (vfolder_store == NULL) {
 		g_warning("Cannot open vfolder store - no vfolders available");
@@ -797,6 +850,17 @@ vfolder_load_storage(void)
 	g_free(storeuri);
 }
 
+void
+vfolder_revert(void)
+{
+	char *user;
+
+	d(printf("vfolder_revert\n"));
+	user = g_strdup_printf("%s/mail/vfolders.xml", mail_component_peek_base_directory (mail_component_peek ()));
+	rule_context_revert((RuleContext *)context, user);
+	g_free(user);
+}
+
 static GtkWidget *vfolder_editor = NULL;
 
 static void
