@@ -148,8 +148,8 @@ static char *filter_size (int size);
 static void folder_changed (CamelObject *o, gpointer event_data, gpointer user_data);
 static void message_changed (CamelObject *o, gpointer event_data, gpointer user_data);
 
-static void hide_save_state(MessageList *ml);
-static void hide_load_state(MessageList *ml);
+static void save_hide_state(MessageList *ml);
+static void load_hide_state(MessageList *ml);
 
 /* note: @changes is owned/freed by the caller */
 /*static void mail_do_regenerate_messagelist (MessageList *list, const char *search, const char *hideexpr, CamelFolderChangeInfo *changes);*/
@@ -1167,17 +1167,15 @@ ml_tree_value_at (ETreeModel *etm, ETreePath path, int col, void *model_data)
 					goto find_colour;
 				}
 			} else if (msg_info->flags & CAMEL_MESSAGE_FLAGGED) {
-				/* FIXME: extract from the xpm somehow. */
+				/* FIXME: extract from the important.xpm somehow. */
 				colour = "#A7453E";
 			} else if ((due_by && *due_by) && !(completed && *completed)) {
 				time_t now = time (NULL);
 				time_t target_date;
 				
 				target_date = camel_header_decode_date (due_by, NULL);
-				if (now >= target_date) {
-					/* FIXME: extract from the xpm somehow. */
+				if (now >= target_date)
 					colour = "#A7453E";
-				}
 			}
 		}
 		return (void *) colour;
@@ -1367,10 +1365,10 @@ static void
 save_tree_state(MessageList *ml)
 {
 	char *filename;
-
+	
 	if (ml->folder == NULL || ml->tree == NULL)
 		return;
-
+	
 	filename = mail_config_folder_to_cachename(ml->folder, "et-expanded-");
 	e_tree_save_expanded_state(ml->tree, filename);
 	g_free(filename);
@@ -1394,7 +1392,7 @@ void
 message_list_save_state (MessageList *ml)
 {
 	save_tree_state (ml);
-	hide_save_state (ml);
+	save_hide_state (ml);
 }
 
 static void
@@ -1625,10 +1623,11 @@ message_list_destroy(GtkObject *object)
 		if (message_list->uid_nodemap) {
 			g_hash_table_foreach(message_list->uid_nodemap, (GHFunc)clear_info, message_list);
 			g_hash_table_destroy (message_list->uid_nodemap);
+			message_list->uid_nodemap = NULL;
 		}
 		
 		save_tree_state(message_list);
-		hide_save_state(message_list);
+		save_hide_state(message_list);
 		camel_object_unhook_event(message_list->folder, "folder_changed", folder_changed, message_list);
 		camel_object_unhook_event(message_list->folder, "message_changed", message_changed, message_list);
 		camel_object_unref (message_list->folder);
@@ -2538,7 +2537,7 @@ message_list_set_folder (MessageList *message_list, CamelFolder *folder, const c
 		
 		while (l) {
 			struct _mail_msg *mm = l->data;
-
+			
 			if (mm->cancel)
 				camel_operation_cancel(mm->cancel);
 			l = l->next;
@@ -2548,7 +2547,7 @@ message_list_set_folder (MessageList *message_list, CamelFolder *folder, const c
 	clear_tree (message_list);
 	
 	if (message_list->folder) {
-		hide_save_state(message_list);
+		save_hide_state(message_list);
 		camel_object_unhook_event((CamelObject *)message_list->folder, "folder_changed",
 					  folder_changed, message_list);
 		camel_object_unhook_event((CamelObject *)message_list->folder, "message_changed",
@@ -2564,7 +2563,6 @@ message_list_set_folder (MessageList *message_list, CamelFolder *folder, const c
 	if (message_list->folder_uri != uri) {
 		g_free(message_list->folder_uri);
 		message_list->folder_uri = g_strdup(uri);
-		message_list->folder = folder;
 	}
 	
 	if (message_list->cursor_uid) {
@@ -2574,6 +2572,9 @@ message_list_set_folder (MessageList *message_list, CamelFolder *folder, const c
 	}
 	
 	if (folder) {
+		camel_object_ref (folder);
+		message_list->folder = folder;
+		
 		/* Setup the strikeout effect for non-trash folders */
 		if (!(folder->folder_flags & CAMEL_FOLDER_IS_TRASH)) {
 			ECell *cell;
@@ -2602,14 +2603,12 @@ message_list_set_folder (MessageList *message_list, CamelFolder *folder, const c
 		camel_object_hook_event (folder, "message_changed",
 					 message_changed, message_list);
 		
-		camel_object_ref (folder);
-		
 		gconf = mail_config_get_gconf_client ();
 		hide_deleted = !gconf_client_get_bool (gconf, "/apps/evolution/mail/display/show_deleted", NULL);
 		message_list->hidedeleted = hide_deleted && !(folder->folder_flags & CAMEL_FOLDER_IS_TRASH);
 		message_list->hidejunk = junk_folder && !(folder->folder_flags & CAMEL_FOLDER_IS_JUNK) && !(folder->folder_flags & CAMEL_FOLDER_IS_TRASH);
 		
-		hide_load_state (message_list);
+		load_hide_state (message_list);
 		mail_regen_list (message_list, message_list->search, NULL, NULL);
 	}
 }
@@ -2664,6 +2663,8 @@ on_selection_changed_cmd(ETree *tree, MessageList *ml)
 {
 	GPtrArray *uids;
 
+	/* not sure if we could just ignore this for the cursor, i think sometimes you
+	   only get a selection changed when you should also get a cursor activated? */
 	uids = message_list_get_selected(ml);
 	g_free(ml->cursor_uid);
 	if (uids->len == 1)
@@ -2671,7 +2672,7 @@ on_selection_changed_cmd(ETree *tree, MessageList *ml)
 	else
 		ml->cursor_uid = NULL;
 
-	if (uids->len <= 1 && !ml->idle_id)
+	if (!ml->idle_id)
 		ml->idle_id = g_idle_add_full (G_PRIORITY_LOW, on_cursor_activated_idle, ml, NULL);
 
 	if (ml->priv->primary_uids) {
@@ -2984,22 +2985,29 @@ message_list_hide_clear (MessageList *ml)
 */
 
 static void
-hide_load_state (MessageList *ml)
+load_hide_state (MessageList *ml)
 {
 	char *filename;
 	FILE *in;
 	guint32 version, lower, upper;
 
+	MESSAGE_LIST_LOCK(ml, hide_lock);
+	if (ml->hidden) {
+		g_hash_table_destroy (ml->hidden);
+		e_mempool_destroy (ml->hidden_pool);
+		ml->hidden = NULL;
+		ml->hidden_pool = NULL;
+	}
+	ml->hide_before = ML_HIDE_NONE_START;
+	ml->hide_after = ML_HIDE_NONE_END;
+
 	filename = mail_config_folder_to_cachename(ml->folder, "hidestate-");
 	in = fopen(filename, "r");
 	if (in) {
 		camel_file_util_decode_fixed_int32 (in, &version);
 		if (version == HIDE_STATE_VERSION) {
-			MESSAGE_LIST_LOCK(ml, hide_lock);
-			if (ml->hidden == NULL) {
-				ml->hidden = g_hash_table_new(g_str_hash, g_str_equal);
-				ml->hidden_pool = e_mempool_new(512, 256, E_MEMPOOL_ALIGN_BYTE);
-			}
+			ml->hidden = g_hash_table_new(g_str_hash, g_str_equal);
+			ml->hidden_pool = e_mempool_new(512, 256, E_MEMPOOL_ALIGN_BYTE);
 			camel_file_util_decode_fixed_int32 (in, &lower);
 			ml->hide_before = lower;
 			camel_file_util_decode_fixed_int32 (in, &upper);
@@ -3013,11 +3021,12 @@ hide_load_state (MessageList *ml)
 					g_hash_table_insert(ml->hidden, uid, uid);
 				}
 			}
-			MESSAGE_LIST_UNLOCK(ml, hide_lock);
 		}
 		fclose(in);
 	}
 	g_free(filename);
+
+	MESSAGE_LIST_UNLOCK(ml, hide_lock);
 }
 
 static void
@@ -3029,7 +3038,7 @@ hide_save_1 (char *uid, char *keydata, FILE *out)
 /* save the hide state.  Note that messages are hidden by uid, if the uid's change, then
    this will become invalid, but is easy to reset in the ui */
 static void
-hide_save_state (MessageList *ml)
+save_hide_state (MessageList *ml)
 {
 	char *filename;
 	FILE *out;
@@ -3099,6 +3108,9 @@ regen_list_regen (struct _mail_msg *mm)
 	CamelMessageInfo *info;
 	int i;
 	
+	if (m->folder != m->ml->folder)
+		return;
+	
 	/* if we have hidedeleted on, use a search to find it out, merge with existing search if set */
 	if (!camel_folder_has_search_capability(m->folder)) {
 		/* if we have no search capability, dont let search or hide deleted work */
